from PyQt5.QtWidgets import QApplication, QTextEdit, QVBoxLayout, QWidget
from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtGui import QTextOption

class ResizableTextEdit(QTextEdit):
    size_changed = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.min_height = 24
        self.min_width = 60
        self.max_height = 260  # Set your maximum height threshold
        self.max_width = 370
        self.text_width = self.min_width
        self.text_height = self.min_height
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setWordWrapMode(QTextOption.NoWrap)
        self.setFixedSize(self.min_width, self.min_height)
        self.document().documentLayout().documentSizeChanged.connect(self.on_size_changed)

    def on_size_changed(self):
        self.adjust_size()
        self.size_changed.emit()

    def adjust_size(self):
        doc_width = self.document().size().width()
        new_width = int(max(doc_width, self.min_width))

        if new_width > self.max_width:
            new_width = self.max_width

        if self.text_width < new_width:
            self.text_width = new_width
            self.setFixedWidth(new_width)

            if doc_width > self.max_width:
                self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
            else:
                self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        doc_height = self.document().size().height()
        new_height = int(max(doc_height, self.min_height))

        if new_height > self.max_height:
            new_height = self.max_height

        if self.text_height < new_height:
            self.text_height = new_height
            self.setFixedHeight(new_height)

            if doc_height > self.max_height:
                self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
            else:
                self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    widget = ResizableTextEdit()
    widget.show()
    sys.exit(app.exec_())