from skimage.draw import ellipsoid_stats

from visualscript.editor.core.views.resizable_text_edit import ResizableTextEdit
from visualscript.editor.core.views.socket_contents.socket_content_widget import SocketContentWidget

class SocketTextContentWidget(SocketContentWidget):
    def __init__(self, model, parent=None):
        self.editText = None
        self.editText_width = 160
        self.padding = 5
        self.initial_text = "None"
        super(SocketTextContentWidget, self).__init__(model, parent)
        self.init_edit_text()

    def load_value(self, value):
        self.editText.setText(value)
        self.update_shape()

    def get_value(self):
        return self.editText.toPlainText()

    def init_edit_text(self):
        self.editText = ResizableTextEdit(self)
        self.editText.setText(self.initial_text)
        self.editText.focusOutEvent = self.onEditTextFocusOutEvent
        self.shape = [self.editText.text_width, self.editText.text_height]
        self.layout.addWidget(self.editText)
        self.update_shape()

    def onEditTextFocusOutEvent(self, event):
        ResizableTextEdit.focusOutEvent(self.editText, event)
        current_text = self.editText.toPlainText()
        if current_text != self.initial_text:
            self.initial_text = current_text
            self.socket.node.update()
            self.socket.node.scene.history.storeHistory("Changed text value to %s" % current_text)

    def update_shape(self):
        grid_size = self.model.parent.scene.graphics.gridSize
        new_height = self.editText.text_height
        if new_height % grid_size:
            new_height += grid_size - new_height % grid_size
        if new_height != self.shape[1]:
            self.shape[1] =  new_height

        new_width = self.editText.text_width
        if new_width % grid_size:
            new_width += grid_size - new_width % grid_size
        if new_width != self.shape[0]:
            self.shape[0] =  new_width

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.shape = [self.width() + self.padding, self.height() + self.padding * 2]
        self.socket.node.graphics.refresh_shape()
        self.socket.node.graphics.update_sockets_pos()
        self.proxy_widget.setPos(-self.width() - self.socket.graphics.radius * 2, -self.height()//2)